@@ -1,19 +1,33 @@
 import asyncio
-from contextlib import asynccontextmanager, contextmanager, suppress
-from typing import AsyncGenerator, Dict, Final, Generator, Hashable, Optional
+from contextlib import AsyncExitStack, asynccontextmanager, suppress
+from typing import Any, AsyncIterator, Dict, Final, Hashable, Optional
 
 __all__ = ('FreqLimit', '__version__')
-__version__ = '0.0.2'
+__version__ = '0.0.5'
 
 import attr
 
 
 @attr.s(auto_attribs=True)
 class Lock:
-    count: int = 0
-    ts: float = -float('inf')
-    lock: asyncio.Lock = attr.ib(factory=asyncio.Lock,
-                                 on_setattr=attr.setters.frozen)
+    ts: float = attr.ib(default=-float('inf'), init=False)
+    _count: int = attr.ib(default=0, init=False)
+    _lock: asyncio.Lock = attr.ib(init=False, factory=asyncio.Lock,
+                                  on_setattr=attr.setters.frozen)
+
+    async def __aenter__(self) -> None:
+        self._count += 1
+        await self._lock.__aenter__()
+
+    async def __aexit__(self, exc_type: Any, exc: Any, tb: Any) -> None:
+        try:
+            await self._lock.__aexit__(exc_type, exc, tb)
+        finally:
+            self._count -= 1
+
+    @property
+    def count(self) -> int:
+        return self._count
 
 
 class FreqLimit:
@@ -32,6 +46,23 @@ class FreqLimit:
         self._clean_task: Optional[asyncio.Task[None]] = None
         self._loop: Final = asyncio.get_running_loop()
 
+    @asynccontextmanager
+    async def acquire(
+        self, key: Hashable = None
+    ) -> AsyncIterator[None]:
+        if self._clean_task is None:
+            self._clean_task = asyncio.create_task(self._clean())
+        if key not in self._locks:
+            self._locks[key] = Lock()
+        async with AsyncExitStack() as stack:
+            stack.callback(self._clean_event.set)
+            await stack.enter_async_context(self._locks[key])
+            delay = self._interval - self._loop.time() + self._locks[key].ts
+            if delay > 0:
+                await asyncio.sleep(delay)
+            self._locks[key].ts = self._loop.time()
+            yield
+
     async def clear(self) -> None:
         if self._clean_task is not None:
             self._clean_task.cancel()
@@ -41,32 +72,6 @@ class FreqLimit:
         self._locks.clear()
         self._clean_event.clear()
 
-    @asynccontextmanager
-    async def acquire(
-        self, key: Hashable = None
-    ) -> AsyncGenerator[None, None]:
-        if self._clean_task is None:
-            self._clean_task = asyncio.create_task(self._clean())
-        if key not in self._locks:
-            self._locks[key] = Lock()
-        with self._count(key):
-            async with self._locks[key].lock:
-                delay = (self._interval - self._loop.time() +
-                         self._locks[key].ts)
-                if delay > 0:
-                    await asyncio.sleep(delay)
-                self._locks[key].ts = self._loop.time()
-                yield
-
-    @contextmanager
-    def _count(self, key: Hashable) -> Generator[None, None, None]:
-        assert key in self._locks
-        self._locks[key].count += 1
-        yield
-        self._locks[key].count -= 1
-        if self._locks[key].count == 0:
-            self._clean_event.set()
-
     async def _clean(self) -> None:
         while True:
             if len(self._locks) == 0:
